Document: kdropbox
Title: Debian kdropbox Manual
Author: <insert document author here>
Abstract: This manual describes what kdropbox is
 and how it can be used to
 manage online manuals on Debian systems.
Section: unknown

Format: debiandoc-sgml
Files: /usr/share/doc/kdropbox/kdropbox.sgml.gz

Format: postscript
Files: /usr/share/doc/kdropbox/kdropbox.ps.gz

Format: text
Files: /usr/share/doc/kdropbox/kdropbox.text.gz

Format: HTML
Index: /usr/share/doc/kdropbox/html/index.html
Files: /usr/share/doc/kdropbox/html/*.html
